#ifndef _DDP_H
#define _DDP_H

#define BIG_ENDIAN_HOST
/* #undef BIG_ENDIAN_HOST */ /* Since the implement of DNA tool is little endian. */

#ifdef CONFIG_DDP_PROTECT
#define CAMEO_DDP_KEY   "CAMEO_DDP"
#endif

/*For packet process*/
#define PACKET_HEADER_OFFSET_IP_VERSION     14
#define IP_VERSION_IP6                      0x6
#define IP_VERSION_IP4                      0x4
#define UDP_PROTOCOL                        0x11

/*For DDP event*/
#define DDP_DATA_RCVD_EVENT                 0x01
#define DDP_GENERAL_REPORT_TMR_EXP_EVENT    0x01<<1
#define DDP_SET_RESET_TMR_EXP_EVENT         0x01<<2

#define DDP_ALL_EVNET                       (DDP_DATA_RCVD_EVENT             | \
                                             DDP_GENERAL_REPORT_TMR_EXP_EVENT | \
                                             DDP_SET_RESET_TMR_EXP_EVENT)

#define DDP_REQUEST             0x0DFD
#define DDP_REPLY               0x0EEE
#define DDP_REPORT              0x0FFF

#define DDP_TRUE                1
#define DDP_FALSE               0

#define DDP_IP4                 0xF000
#define DDP_IP6                 0x0000

/*OP code definition*/
#define DDP_GENERAL_REPORT_OP       0x00A0 /*160*/
#define DDP_DISCOVERY_OP            0x00A1 /*161*/
#define DDP_SET_BASIC_INFO_OP       0x00A2 /*162*/
#define DDP_USER_VERIFY_OP          0x00A3 /*163*/
#define DDP_QUERY_SUPPORT_OPT_OP    0x00A5 /*165*/
#define DDP_RESET_OP                0x00AA /*170*/
#define DDP_FACTORY_RESET_OP        0x00AB /*171*/
#define DDP_INFO_OP                 0x00E0 /*224*/
#define DDP_SET_IP4_ADDRESS_OP      0x00E1 /*225*/
#define DDP_FW_UPGRADE_OP           0x0100 /*256*/
#define DDP_CFG_BACKUP_OP           0x0101 /*257*/
#define DDP_CFG_RESTORE_OP          0x0102 /*258*/
#define DDP_REBOOT_OP               0x0103 /*259*/

/*DDP Request body length*/
#define DDP_GENERAL_REPORT_REQ_LEN      295
#define DDP_DISCOVERY_REQ_LEN           0
/*#define DDP_SET_BASIC_INFO_REQ_LEN_v1   217*/
#define DDP_SET_BASIC_INFO_REQ_LEN      283 /*155+128*/
#define DDP_USER_VERIFY_REQ_LEN         128
#define DDP_QUERY_SUPPORT_OPT_REQ_LEN   0 /*For version 1: 128, for version 2: 0*/
#define DDP_RESET_REQ_LEN               (128 + 4)
#define DDP_FACTORY_RESET_REQ_LEN       (128 + 4)
#define DDP_INFO_REQ_LEN                0
#define DDP_SET_IP4_ADDRESS_REQ_LEN_SYS (14 + 128)
#define DDP_SET_IP4_ADDRESS_REQ_LEN_DNS (10 + 128)
#define DDP_FW_UPGRADE_REQ_LEN_IP4      (135 + 128)
#define DDP_FW_UPGRADE_REQ_LEN_IP6      (147 + 128)
#define DDP_FW_UPGRADE_STATUS_REQ_LEN   (1 + 128)
#define DDP_CFG_BACKUP_REQ_LEN_IP4      DDP_FW_UPGRADE_REQ_LEN_IP4
#define DDP_CFG_BACKUP_REQ_LEN_IP6      DDP_FW_UPGRADE_REQ_LEN_IP6
#define DDP_CFG_BACKUP_STATUS_REQ_LEN   DDP_FW_UPGRADE_STATUS_REQ_LEN
#define DDP_CFG_RESTORE_REQ_LEN_IP4     DDP_FW_UPGRADE_REQ_LEN_IP4
#define DDP_CFG_RESTORE_REQ_LEN_IP6     DDP_FW_UPGRADE_REQ_LEN_IP6
#define DDP_CFG_RESTORE_STATUS_REQ_LEN  DDP_FW_UPGRADE_STATUS_REQ_LEN
#define DDP_REBOOT_REQ_LEN              (128 + 4)
/*If DDP request has different version or different case, which may need to accept different length
  of request DDP packet, define request length as DDP_DIFFERENT_DATA_LEN*/
#define DDP_DIFFERENT_DATA_LEN          65535

/*DDP Reply body length*/
#define DDP_GENERAL_REPORT_REPLY_LEN        295
/*#define DDP_DISCOVERY_REPLY_LEN_v1          312*/
#define DDP_DISCOVERY_REPLY_LEN             483
#define DDP_SET_BASIC_INFO_REPLY_LEN        16
#define DDP_USER_VERIFY_REPLY_LEN           4
#define DDP_QUERY_SUPPORT_OPT_REPLY_LEN     (DDP_END + 1) * sizeof(unsigned short) /*Depend on supported function no.#*/
#define DDP_RESET_REPLY_LEN                 16
#define DDP_FACTORY_RESET_REPLY_LEN         16
#define DDP_INFO_REPLY_LEN                  16
#define DDP_SET_IP4_ADDRESS_REPLY_LEN       129
#define DDP_FW_UPGRADE_REPLY_LEN            130
#define DDP_CFG_BACKUP_REPLY_LEN            130
#define DDP_CFG_RESTORE_REPLY_LEN           130
#define DDP_REBOOT_REPLY_LEN                16

/*Request mode & Ret code*/
#define DDP_REQUEST_ALL         0x0000
#define DDP_REQUEST_SPECIFIC    0x7777

#define DDP_REPLY_SUCCESS       0x0
#define DDP_REPLY_FAILURE       0x1 /*The further detail error should refer to error setting number
                                      field in the setting result structure*/
#define DDP_REPLY_AUTH_FAILURE  0x2
#define DDP_REPLY_NOT_SUPPORT   0x3

#define DDP_REPLY_NONE          0x4 /*Do not reply any packet*/

/*Error code, use to fill in tDdpResult.u4ErrCode*/
#define DDP_ALL_FIELD_SUCCESS               0  /* All fields in the command are correctly committed to Device. */
#define DDP_INVALID_FIELD_NAME              1  /* Set_Basic_Settings: Device Name is error */
#define DDP_INVALID_FIELD_IP                2  /* Set_Basic_Settings: Device IP is error */
#define DDP_INVALID_FIELD_PORT              3  /* Set_Basic_Settings: Web Port Number is error */
#define DDP_INVALID_FIELD_NETMASK           4  /* Set_Basic_Settings: Subnet Mask is error */
#define DDP_INVALID_FIELD_GATEWAY           5  /* Set_Basic_Settings: Default Gateway is error */
#define DDP_INVALID_FIELD_PRIMARY_DNS       6  /* Set_Basic_Settings: Primary DNS is error */
#define DDP_INVALID_FIELD_SECONDARY_DNS     7  /* Set_Basic_Settings: Secondary DNS is error */
#define DDP_INVALID_FIELD_DHCP              8  /* Set_Basic_Settings: DHCP is error */

#define DDP_INVALID_FIELD_DATETIME          101 /* Set_Systime_Datetime: Date-time is error */
#define DDP_INVALID_FIELD_TIMEZONE          102 /* Set_Systime_Datetime: Time-zone is error */
#define DDP_INVALID_FIELD_DL_SAVING         103 /* Set_Systime_Datetime: Daylight Saving Time is error */
#define DDP_INVALID_FIELD_NTP_ENABLE        104 /* Set_Systime_Datetime: NTP Enabled is error */
#define DDP_INVALID_FIELD_NTP_SERVER        105 /* Set_Systime_Datetime: NTP Server is error */

#define DDP_INVALID_FIELD_WIRELESS_ENABLE       201 /* Set_Wireless, Wireless_Simple_Setting : Enable Wireless is error */
#define DDP_INVALID_FIELD_CHANNEL               202 /* Set_Wireless, Wireless_Simple_Setting: Wireless Channel is error */
#define DDP_INVALID_FIELD_ESSID                 203 /* Set_Wireless, Wireless_Simple_Setting: SSID is error */
#define DDP_INVALID_FIELD_WIRELESS_MODE         204 /* Set_Wireless, Wireless_Simple_Setting: Wireless Mode is error */
#define DDP_INVALID_FIELD_WIRELESS_ENCRYPTION   205 /* Set_Wireless, Wireless_Simple_Setting: Encryption is error */
#define DDP_INVALID_FIELD_WIRELESS_AUTH         206 /* Set_Wireless, Wireless_Simple_Setting: Authentication is error */
#define DDP_INVALID_FIELD_WIRELESS_KEY_LEN      207 /* Set_Wireless: WEP Key Length Mode s is error */
#define DDP_INVALID_FIELD_WIRELESS_KEY_FORMAT   208 /* Set_Wireless: WEP Key Format is error */
#define DDP_INVALID_FIELD_WIRELESS_TX_RATE      209 /* Set_Wireless: TX Rate is error */
#define DDP_INVALID_FIELD_WIRELESS_WPA_KEY      210 /* Set_Wireless: WPA Key is error */
#define DDP_INVALID_FIELD_WIRELESS_WEP_KEY      211 /* Set_Wireless: One of WEP Key1, WEP Key2, WEP Key3, WEP Key4 is error */
#define DDP_INVALID_FIELD_WIRELESS_ KEY         212 /* Wireless_Simple_Setting: Field Key is error */

#define DDP_INVALID_FIELD_PPPOE_ENABLE          301 /* Set_Advance_Network: PPPoE Setting is error */
#define DDP_INVALID_FIELD_PPPOE_USERNAME        302 /* Set_Advance_Network: PPPoE User's ID is error */
#define DDP_INVALID_FIELD_PPPOE_PASSWORD        303 /* Set_Advance_Network: PPPoE User's Password is error */
#define DDP_INVALID_FIELD_DDNS_INDEX            304 /* Set_Advance_Network: Set DDNS Provider Index is error */
#define DDP_INVALID_FIELD_DDNS_USERNAME         305 /* Set_Advance_Network: DDNS User's ID is error */
#define DDP_INVALID_FIELD_DDNS_PASSWORD         306 /* Set_Advance_Network: DDNS User's Password is error */
#define DDP_INVALID_FIELD_DDNS_HOSTNAME         307 /* Set_Advance_Network: DDNS Server is error */

#define DDP_INVALID_FIELD_CHANGEID_ADMIN                    1001 /* Change_ID: The name of administrator's account can't be
                                                                    changed. */
#define DDP_INVALID_FIELD_CHANGEID_ADMIN_LENGTH             1002 /* Change_ID: The length of of administrator's name is
                                                                    too long. Meanwhile the maximum character is stored in
                                                                    field Extension of Execution_Status structure */
#define DDP_INVALID_FIELD_CHANGEID_ADMIN_ILLEGAL_CHAR       1003 /* Change_ID: The proposed value of administrator's name
                                                                    contains illegal character. */
#define DDP_INVALID_FIELD_CHANGEID_PASSWORD                 1011 /* Change_ID: The password of administrator's account
                                                                    can't be changed. */
#define DDP_INVALID_FIELD_CHANGEID_PASSWORD_LENGTH          1012 /* Change_ID: The length of of administrator's password
                                                                    is too long. Meanwhile the maximum character is stored in
                                                                    field Extension of Execution_Status structure */
#define DDP_INVALID_FIELD_CHANGEID_PASSWORD_ILLEGAL_CHAR    1013 /* Change_ID: The proposed value of administrator's
                                                                    password contains illegal character. */

#define DDP_INVALID_FIELD_SNMP_VIEWNAME_LOST	            2001 /*SNMP_Configuration: Lack of SNMP view name. This error
                                                                   number is used to notify the server that the SNMP view
                                                                   name must be configured in the request packet.*/

#define DDP_INVALID_FIELD_DUPLICATED_ENTRY	                65534 /*General: This return number is used to notify server
                                                                    that this entry had been created.*/

#define DDP_INVALID_FIELD_SERVER_MESSAGE_TYPE_ERROR	        65535 /*General: This return number is used to notify server
                                                                    that client can not understand this format.*/

/*DDP operation status, use to fill in tDdpResult.u4OperStatus*/
#define DDP_OPERATION_DONE              0
#define DDP_OPERATION_RENEWAL           1
#define DDP_OPERATION_WAIT              2

/*Code length. Notice: for base 64 MIME, length should be 4*N */
#define DDP_ID_ENCODE_LEN               64
#define DDP_PSW_ENCODE_LEN              64

/*DDP data start offset*/
#define DDP_IP4_BUFFER_START_OFFSET     42 /*DATA_LINK_HDR_LEN(14) + IPv4_HEADER_LEN(20) + UDP_HDR_LEN(8) */
#define DDP_IP6_BUFFER_START_OFFSET     62 /*DATA_LINK_HDR_LEN(14) + IPv6_HEADER_LEN(40) + UDP_HDR_LEN(8)*/

/*Packet header length*/
#define DATA_LINK_HDR_LEN               14
#define IP4_HDR_LEN                     20
#define IP6_HDR_LEN                     40
/*DDP header length*/
#define DDP_IP4_HEADER_LEN              22  /*sizeof(tDdpHeader) with IPv4*/
#define DDP_IP6_HEADER_LEN              34  /*sizeof(tDdpHeader) with IPv6*/
#define DDP_EXE_RESULT_LEN              16  /*sizeof(tDdpResult)*/

/*Current DDP version*/
#define DDP_CURRENT_VERSION             0x2
#define DDP_CURRENT_SPEC_VERSION        "0.24"

/*DDP return value*/
#define DDP_SUCCESS                     0
#define DDP_FAILURE                     1

/*---DDP General Definition---*/
#define DDP_ENABLE                      1
#define DDP_DISABLE                     0

/* Basic Settings. */
#define DDP_DHCP_ENABLE                 0x01
#define DDP_DHCP_DISABLE                0x00
/* end of Basic Settings. */

/* Query Neighbor Information */
#define MAX_DDP_NEIGHBOR_DEV_INFO_COUNT     10
#define DDP_NEIGHBOR_AGE_TIME               3 /*The range of age time is (DDP_NEIGHBOR_AGE_TIME * 30 secs)*/
/* End of Query Neighbor Information */

/* Vlan Configuration. */
#define DDP_VLAN_NAME_LEN                   32
#define DDP_PORT_LIST_SIZE                  64
#define DDP_PORT_LIST_TYEP_STRING           0
#define DDP_PORT_LIST_TYEP_BITMAP           1
#define DDP_PORT_LIST_TYEP_STRING_RESERVED  "All"

/*---ASV configuration Macro Definition---*/
#define DDP_SURVEILLANCE_VLAN_STATE_DISABLE 0
#define DDP_SURVEILLANCE_VLAN_STATE_ENABLE  1
#define DDP_SURVEILLANCE_VLAN_OP_ADD        0   /* Operate : Add. */
#define DDP_SURVEILLANCE_VLAN_OP_DEL        1   /* Operate : Del. */
#define DDP_SURVEILLANCE_VLAN_USRDEF_MAC    0   /* User config. MAC. */
#define DDP_SURVEILLANCE_VLAN_USRDEF_OUI    1   /* User config. OUI. */

#define DDP_SURVEILLANCE_VLAN_COMPONENT_TYPE_VMS_SERVER  0  /* Video Management Server (VMS). */
#define DDP_SURVEILLANCE_VLAN_COMPONENT_TYPE_VMS_CLIENT  1  /* VMS client. */
#define DDP_SURVEILLANCE_VLAN_COMPONENT_TYPE_VE          2  /* Video encoder. */
#define DDP_SURVEILLANCE_VLAN_COMPONENT_TYPE_NS          3  /* Network storage. */
#define DDP_SURVEILLANCE_VLAN_COMPONENT_TYPE_OISD        4  /* Other IP Surveillance device. */

#define DDP_PARAMETER_SETTING           1
#define DDP_STATUS_QUERY                2

#define UDP_HDR_LEN                     8

#define DDP_READ_OFFSET(pPktBuf)        (pPktBuf->readoffset)
#define DDP_WRITE_OFFSET(pPktBuf)       (pPktBuf->writeoffset)

/* Please define/undefine BIG_ENDIAN_HOST on the top of this file */
#ifdef BIG_ENDIAN_HOST
#define DDP_NTOHL(x) (unsigned int) (((x & 0xFF000000) >> 24) | \
                                     ((x & 0x00FF0000) >> 8)  | \
                                     ((x & 0x0000FF00) << 8 ) | \
                                     ((x & 0x000000FF) << 24))
#define DDP_NTOHS(x) (unsigned short) (((x & 0xFF00) >> 8) | ((x & 0x00FF) << 8))
#define DDP_HTONL(x) (unsigned int) (DDP_NTOHL(x))
#define DDP_HTONS(x) (unsigned short) (DDP_NTOHS(x))
#else
#define DDP_NTOHL(x) (unsigned int) (x)
#define DDP_NTOHS(x) (unsigned short) (x)
#define DDP_HTONL(x) (unsigned int) (x)
#define DDP_HTONS(x) (unsigned short) (x)
#endif /* end of BIG_ENDIAN_HOST */

#define DDP_IP4_ADDR_NTOHL(x) DDP_NTOHL(x)
#define DDP_IP4_ADDR_HTONL(x) DDP_HTONL(x)

#define DDP_CONVERT_IPADDR_TO_STR(pString, u4Value)  \
{    \
    unsigned int      IpAddr;                       \
    IpAddr = htonl (u4Value);                       \
    memset(pString, 0, sizeof(pString));            \
    sprintf(pString, "%u.%u.%u.%u",                 \
                    ((IpAddr & 0xFF)),              \
                    ((IpAddr >> 8) & 0xFF),         \
                    ((IpAddr >> 16) & 0xFF),        \
                    (IpAddr >> 24) & 0xFF);         \
}

/*OP code definition*/
enum
{
    DDP_GENERAL_REPORT = 0,
    DDP_DISCOVERY,
    DDP_SET_BASIC_INFO,
    DDP_USER_VERIFY,
    DDP_QUERY_SUPPORT_OPT,
    DDP_RESET,
    DDP_FACTORY_RESET,
    DDP_INFO,
    DDP_SET_IP4_ADDRESS,
    DDP_FW_UPGRADE,
    DDP_CFG_BACKUP,
    DDP_CFG_RESTORE,
    DDP_REBOOT,
    DDP_END
};

enum
{
    DDP_OP_CODE = 0,
    DDP_REQ_LEN,
    DDP_RLY_LEN,
    DDP_AUTH_WANT,
    DDP_OP_ENABLE_FLAG
};

enum
{
    /*DDP Firmware upgrade status definition*/
    DDP_FIRMUPGRADE_IDLE = 0,
    DDP_FIRMUPGRADE_SETTING,
    DDP_FIRMUPGRADE_FILE_TRANS_DONE,
    DDP_FIRMUPGRADE_PROCESSING,
    DDP_FIRMUPGRADE_FINISH,
    DDP_FIRMUPGRADE_FAILED,
    /*DDP Firmware upgrade result definition*/
    DDP_FIRMUPGRADE_SUCCESS,
    DDP_FIRMUPGRADE_FAILURE,
    DDP_FIRMUPGRADE_FAILURE1,       /*DDP_FW_WRONG_FILE*/
    DDP_FIRMUPGRADE_FAILURE2,       /*DDP_FW_TRANSFER_FAILED*/
    DDP_FIRMUPGRADE_FAILURE3        /*DDP_FW_WRONG_CHECKSUM*/
};

enum
{
    /*DDP configuration backup/restore status Definition*/
    DDP_CFGPROCESS_IDLE = 0,
    DDP_CFGPROCESS_SETTING,
    DDP_CFGPROCESS_FILE_TRANS_DONE,
    DDP_CFGPROCESS_PROCESSING,
    DDP_CFGPROCESS_FINISH,
    DDP_CFGPROCESS_FAILED,
    DDP_CFGPROCESS_FAILED1 /*DDP_CFG_WRONG_FILE, WRONG_CHECKSUM*/
};

/*
 * This defines the structure of an UDP header.
 * The header is extracted on the arrival of each message.
 */
typedef struct
{
    unsigned short     u2Src_u_port;
    unsigned short     u2Dest_u_port;
    unsigned short     u2Len;
    unsigned short     u2Cksum;
} t_UDP;

/*
 * IP Header structure. Used for extracting parameters from buffer
 * by directly typecasting to this structure.
 */
typedef struct
{
    unsigned char      u1Ver_hdrlen;  /* Version + Header Length */
    unsigned char      u1Tos;         /* Type of service */
    unsigned short     u2Totlen;      /* Total length  IP header + DATA */
    unsigned short     u2Id;          /* Identification */
    unsigned short     u2Fl_offs;     /* Flags + fragment offset */
    unsigned char      u1Ttl;         /* Time to live */
    unsigned char      u1Proto;       /* Protocol */
    unsigned short     u2Cksum;       /* Checksum value */
    unsigned int       u4Src;         /* Source address */
    unsigned int       u4Dest;        /* Destination address */
    unsigned char      u1Options[4];  /* Options field */
} t_IP_HEADER;

typedef struct
{
    char *pdata;
    unsigned int len;
    unsigned int readoffset;
    unsigned int writeoffset;
} tDdpPktBuffer;

/*Packet header structure, for packet creation*/
typedef struct
{
    unsigned int    src;
    unsigned int    dst;
    unsigned char   mbz;
    unsigned char   protocol;
    unsigned short  length;
    unsigned short  checksum;
}__attribute__((__packed__)) Pseudo_HDR;

/* DDP header structure */
typedef struct
{
    unsigned short u2Identifier;
    unsigned short u2Sequence;
    unsigned short u2OPCode;
    unsigned char a1MacAddr[6];
    union
    {
        unsigned int u4Ipv4Addr;
        unsigned char au1Iv6Addr[16];
    } ip_addr;
    unsigned short u2Mode;
    unsigned short u2PVer;
    unsigned short u2Length;
} tDdpHeader;

/*ID & Password structure*/
typedef struct
{
    unsigned char   au1Id[64];
    unsigned char   au1Psw[64];
} tDdpIdPsw;

/*Setting result (execution status) structure*/
typedef struct
{
    unsigned int    u4ErrCode;
    unsigned int    u4OperStatus;
    unsigned int    u4WaitTime;
    unsigned int    u4Extension;

} tDdpResult;

/*Device setting structure*/
typedef struct
{
    unsigned char   au1DeviceName[64];      /* Device Name, BF_TEXT, 64 Byte.*/
    unsigned int    u4Ipv4Addr;             /* Device IP, BF_BINARY, 4 Byte. */
    unsigned short  u2WebPort;              /* Web Port Number, BF_BINARY, 2 Byte. */
    unsigned char   u1Reserved;             /* Reserved, BF_BINARY, 1 Byte. */
    unsigned char   u1Ipv4DnsSpec;          /* Specify Customized DNS, BF_BINARY, 1 Byte. */
    unsigned int    u4Ipv4SubMask;          /* Subnet Mask, BF_BINARY, 4 Byte. */
    unsigned int    u4Ipv4GateWay;          /* Default Gateway, BF_BINARY, 4 Byte. */
    unsigned int    u4Ipv4Dns1st;           /* Primary DNS, BF_BINARY, 4 Byte. */
    unsigned int    u4Ipv4Dns2ed;           /* Secondary DNS, BF_BINARY, 4 Byte. */
    unsigned char   u1Dhcp;                 /* DHCP, BF_BINARY, 1 Byte. */
    /*---------------------*/
    unsigned char   au1Ipv6Addr[16];        /* Device IPv6 address, BF_BINARY, 16 Byte. */
    unsigned char   u1Ipv6Prefix;           /* Device IPv6 prefix, BF_BINARY, 1 Byte. */
    unsigned char   au1Ipv6GateWay[16];     /* Default gateway IPv6 address, BF_BINARY, 16 Byte. */
    unsigned char   au1Ipv6Dns1st[16];      /* Primary DNS IPv6 address, BF_BINARY, 16 Byte. */
    unsigned char   au1Ipv6Dns2ed[16];      /* Secondary DNS IPv6 address, BF_BINARY, 16 Byte. */
    unsigned char   u1Ipv6Dhcp;             /* DHCP v6 state, BF_BINARY, 1 Byte. */
} tDdpSystemInfo_ver2;

typedef struct
{
    unsigned char   au1ProductName[64];         /* Product Name, BF_TEXT, 64 Byte.*/
    unsigned char   au1ProductCategory[32];     /* Product Category, BF_TEXT, 32 Byte.*/
    unsigned char   au1Version[32];             /* Version, BF_TEXT, 32 Byte.*/
    unsigned char   a1MacAddr[6];               /* Mac Address, BF_BINARY, 6 Byte. */
    unsigned char   au1HarewareVersion[32];     /* Hareware Version, BF_TEXT, 32 Byte.*/
    unsigned char   au1SerialNumber[32];        /* Serial Number, BF_TEXT, 32 Byte.*/
    unsigned char   au1SystemName[64];          /* System Name, BF_TEXT, 64 Byte.*/
    unsigned char   u4DevIPAddr[4];                /* Device IP, BF_BINARY, 4 Byte. */
    unsigned char   u4Ipv4SubMask[4];              /* Subnet Mask, BF_BINARY, 4 Byte. */
    unsigned char   u2PortInterfaceNumber[8];   /* Port Interface Number, BF_BINARY, 8 Byte. */
    unsigned char   au1Ipv6Addr[16];            /* Device IPv6 address, BF_BINARY, 16 Byte. */
    unsigned char   u1Ipv6Prefix;               /* Device IPv6 addr prefix if support, BF_BINARY, 1 Byte.*/
} tDdpGeneralReport;

/*FW upgrade and Configuration restore/backup information structure */
typedef struct
{
    unsigned char   u1SubServerType; /*1: "Parameter configure" is used to deliver all download process
                                  related parameter to client. Client can check these parameters
                                  are correct or not.
                               2: "Status query" is used to sync download status between DDP
                                  server and DDP client. It can help DDP server make sure current
                                  downloading */
    unsigned char   u1ServerType;    /*1: TFTP server.  2: HTTP server.*/
    unsigned char   u1ServerIpType;  /*1: IPv4.  2: IPv6.*/
    union
    {
        unsigned int    u4Ipv4Addr;
        unsigned char   au1Iv6Addr[16];
    } ip_addr;
    unsigned char   au1FileName[128]; /*For TFTP, this field will indicate the name of the configuration
                                file which will be downloade. For HTTP, this field will indicate
                                the URL address which will be connected.*/
} tDdpUpgradeInfo;

/*FW upgrade and Configuration restore/backup reply structure*/
typedef struct
{
    unsigned char   u1SubServerType; /*1: Parameters configure. 2: Download status query.*/
    unsigned char   u1ReplyStatus;   /*0: Success.  1: Fail.  2: under processing.  3: process done.*/
    unsigned char   au1FailReason[128]; /*If status is fail, use this filed to report the fail reason.*/
} tDdpUpgradeReply;


void DDPhmacInit();
char DDPHeaderProcess(tDdpPktBuffer *pPktBuffer);
char DDPAuthProcess(tDdpPktBuffer *pPktBuf);
char DDPOPCodeProcess(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);
char DDPDoDiscover(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);
char DDPDoSetBasicInfo(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);
char DDPDoUserVerify(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);
char DDPDoQuerySupportOpt(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);
char DDPDoReset(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);
char DDPDoFwUpgrade(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);
char DDPDoCfgProcess(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);
char DDPDoReboot(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);
char DDPDoInfo(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);
char DDPDoSysIp4Set(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);
char DDPDoDeviceInfoSet(tDdpHeader *pDdpInterface, tDdpPktBuffer *pPktBuf);

char DDPForwardPacket(tDdpPktBuffer *pPktBuf);
char DDPSend(tDdpPktBuffer *pPktBuf, tDdpHeader *pDdpInterface,  unsigned short u2RetCode, unsigned char *pDdpOutput, unsigned int u4DataSize);
void *DDPBufRead(tDdpPktBuffer *pPktBuf, unsigned char *pData, unsigned int u4Offset, unsigned int u4Size);
tDdpPktBuffer *DDPBufAllocate(int size);
void DDPBufRelease(tDdpPktBuffer *pPktBuf);
unsigned char *DDPBufGetDataPtr(tDdpPktBuffer *pPktBuf, unsigned int u4Offset, unsigned int u4Units);
tDdpPktBuffer *DDPBufDuplicate(tDdpPktBuffer *pPktBuf);
void DDPBufCopy(tDdpPktBuffer *pPktBuf, unsigned char *pData, unsigned int u4Offset, unsigned int u4Size);
char DDPCheckMAC(unsigned char *pMacAddr);
char DDPCheckIP(tDdpHeader *pDdpInterface);
unsigned short CalcIPCheckSum(tDdpPktBuffer *pPktBuffer, unsigned int u4Size, unsigned int u4Offset);
unsigned short CalcUDPCheckSum(unsigned short *dp, unsigned short length);
char Decryption_Base64Mime(unsigned char *au1DecodeArr, unsigned int u4Size, unsigned char *au1DataArr);

#ifdef CONFIG_DDP_PROTECT
extern void hmacmd5(unsigned char *input, int inputlen, unsigned char *key, int keylen, unsigned char output[16]);
#endif

#endif /* end of !_DDP_H */
